*DIAM 2.5
*(C) Minh Ha-Duong, CIRED, 1997-2002,           minh.ha.duong@cmu.edu
*Copyleft according to the GNU GPL v. 2 or later
*Thanks to: T. Chapuis M. Grubb A. Manne J. Singer P. Dumas D. Keith
$TITLE $Id: diam.gms,v 1.42 2003/03/30 15:39:11 haduong Exp haduong $

SETS
T             Time periods             /1*30/; ALIAS (T, U)

VARIABLE
J           Minimisation objective
I(T)        Climate change impact (tera USD)
E(T)        Net emissions (GtC);

POSITIVE VARIABLES
M(T)        CO2 concentration (parts per million)
R(T)        CO2 stored  (GtC)
X(T)        Abatement level (fraction of reference fossil emissions)
CX(T)       Abatement costs (tera USD)
DX(T)       First time difference of abatement level for X
Z(T)        Abatement by backstop tech (fraction of reference fossil emissions)
CZ(T)       Backstop costs (tera USD)
DZ(T)       First time difference of Z;

SCALARS
CZI           Initial carbon sequestration cost (USD by tC) /75/
TauZ          Rate of exogenous decline in above cost       /0.01/
CXI           Initial reduction costs factor                /0.0245/
TauX          Rate of exogenous decline in reduction costs  /0.01/
AdapRate      Rate of exogenous decline in damages          /0.01/
WealthI       Global initial consumption (tera USD)         /18/
D             Characteristic time of energy systems         /50/
DLag          Damage lag                                    /2/
DiscRate      Utility discount rate                         /0.01/
NonLinDa      Non-Linear climate change                     /0.04/
epsilon       Sharpness of the damage curve                 /0.1/
C2x           Damage at 2xC02 (% global cons.)              /2.0/
Tresh         Damages nonlinearity treshold (ppmv)          /550/
Ceiling       Damages nonlinearity ceiling (ppmv)           /650/
Growth        Global growth (annual % rate)                 /1.5/
NetAbatM      Maximum relative net abatement                /3/
leakrate      CO2 reservoir leak rate                       /0.005/
Epenalty      Energy penalty of carbon capture              /0.15/ ;

PARAMETERS
Wealth(T)     Global consumption (tera US$ 1990)
Date(T)       Year corresponding to period T
EmisRef(T)    Reference emissions (fossil)
EmisLand(T)   Reference emissions (land use)
ConcRef(T)    Reference concentrations
AtmPRep(T,U)  Atmospheric perturbation CO2 response function  ;

Wealth(T) = WealthI * power(1+Growth/100, 10*(ORD(T)-4) )  ;

*Tables defining Date, EmisRef, EmisLand, ConcRef and AtmPRep
$include ccycle.gms

EQUATIONS
eqJ          Objective function : Expected total discounted felicity
eqCX(T)      Cost of abatement (quadratic part)
eqCZ(T)      Cost of backstop
eqDX(T)
eqDZ(T)
eqI(T)       Impact function with threshold
eqM(T)       Linear impulse response perturbation carbon cycle
eqR(T)       Storage in leaky reservoir
eqE(T)       Emissions (net of air capture)
eqZ(T)       Bound on total abatement
;

eqJ.. J =E=  SUM( (T)$(ORD(T) GE 5),
    - 2 * power(1 - DiscRate, 10*ORD(T)) /
    sqrt( Wealth(T) - I(T) - CX(T) - CZ(T) ) )  ;

eqDX( T)$(ORD(T) GE 5)..   10 * DX(T) =E= X(T) - X(T-1);
 
eqDZ(T)$(ORD(T) GE 5)..    10 * DZ(T) =E= Z(T) - Z(T-1);

eqCX( T)$(ORD(T) GE 5)..
 CX( T) =E= 
  CXI * WealthI * EmisRef(T)/EmisRef("4") / power(1+TauX, 10*(ORD(T)-4)) *
  ( X(T)*X(T) + D*D*DX(T)*DX(T) )  ;

eqCZ( T)$(ORD(T) GE 5)..
 CZ( T) =E= CZI / 1000 / power(1+TauZ, 10*(ORD(T)-4) ) *
             Z(T) * EmisRef(T) * ( 1 + power(D * DZ(T), 2) )  ;

eqI( T)$(ORD(T) GE 5)..
 I( T) =E= 
  (C2x/100) * ( Wealth(T) / power(1 + AdapRate, 10*(ORD(T)-4) ) ) *
  (M( T-DLag) - 335) / (550-335)
  +
  NonLinDa * Wealth(T) / (1 +
   exp(log ((2-epsilon)/epsilon) *
       ( (Tresh+Ceiling - 2*M(T-DLag)) / (Ceiling - Tresh) ) ) );

eqM( T)$(ORD(T) GE 5)..
 M( T) =E= ConcRef(T) -
 10 / 2.123 * SUM(U$(ORD(U) LT ORD(T)+1), AtmPRep(T, U) *
                   (EmisRef(U)*(X( U) + Z(U)) - R(T)*leakrate ));
* The alternate expression below is shorter to write
* but will GAMS eliminate E before passing to the solver?
*                   (EmisRef(U) - E(U) ));

eqR(T)$(ORD(T) GE 5)..   
  R(T) =E= R(T-1) * (1 - 10*leakrate) + 10 * Z(T-1) * EmisRef(T-1) /(1-Epenalty);

eqE(T)$(ORD(T) GE 3)..
   E(T) =E= EmisRef(T) * (1 - X(T) - Z(T) ) + R(T)*leakrate ;

eqZ(T)$(ORD(T) GE 5)..
   X(T) + Z(T) =L= NetAbatM ;

*SENSIBLE BOUND STARTING POINT and CONSTRAINTS

M.UP(T)  = 750; M.L(T) = ConcRef(T);     M.LO(T) = 275;
I.UP(T)  =   5; I.LO(T) = 0;
X.UP(T)  = 1.0; X.L(T) = (ORD(T)-5)/30;
Z.UP(T)  = 2.0; Z.L(T) = (ORD(T)-5)/90;
E.UP(T)  =  30; E.LO(T)= -20;
R.UP(T)  = 5000; 
CX.UP(T) =  10;
CZ.UP(T) =  10;
DX.UP(T) = 0.02;
DZ.UP(T) =  1;

M.FX("1") = 317; X.FX( "1") = 0.0; Z.FX("1") = 0.0; R.FX("1") = 0.0;
M.FX("2") = 335; X.FX( "2") = 0.0; Z.FX("2") = 0.0; R.FX("2") = 0.0;
M.FX("3") = 354; X.FX( "3") = 0.0; Z.FX("3") = 0.0; R.FX("3") = 0.0;
M.FX("4") = 374; X.FX( "4") = 0.0; Z.FX("4") = 0.0; R.FX("4") = 0.0;


MODEL DIAM  /ALL/;
diam.optfile=1;
*option nlp=kestrel;
*option nlp=minos;
option nlp=conopt
SOLVE DIAM MAXIMIZING J USING NLP;
